
                      PROGRAMMER'S MANUAL


OVERVIEW:

The B-Tree program implements a data base of key words.  The key
words are members of text documents, each of which has been
concatenated to a main Docs file.  A key may contain up to
MAX_WORD_SIZE characters. 

The program creates and maintains a B-Tree with the property that 
all keys are stored, with their relative data, in the leaves of 
the tree, and all intermediate pages contain pointers which lead
to the leaves, i.e., all successful searching will lead to a leaf.
The Root is always stored as the first page.

To insert (index) a document in the tree, the program work as follows:

(1)  Concatenate the new text document to the main Docs file.

(3)  Upon reading a key word from the text document, determine 
     if the B-Tree exists or does not exist.

       .If the B-Tree does not exist, create the Root, create the first  
        Leaf page, create a Postings record in the Postings file (this
        Postings record will contain the key's offset within the Docs
        file).  Insert into the B-Tree the key and the Postings record 
        offset. 
    
       .If the B-Tree does exist, search the B-Tree to determine if the  
        key is already inserted or not.  

          .If the key is not in the B-Tree, create a Postings record
           (again, this Postings record will contain the key's offset
           within the Docs file).  Insert into the B-Tree the key and
           the Postings record offset.  

          .If the key is already in the B-Tree, update the Postings
           record to contain the new offset within the Docs file.

     Repeat until document EOF (text file).


FILES:

The files that are accessed and utilized are as follows:

(1)  '*fp_input' is a pointer to the new document file that is given
     at execution time.  This text file will be concatenated to the
     main Docs file.

(2)  '*fp_docs' is a pointer to the main Docs file which is a composite
     of concatenated documents.

(3)  '*fp_postings' is a pointer to the Postings file.  The Postings 
     file consists of Postings records, each of which contains offsets 
     into the main Docs file.  There is a maximum number of offsets,
     MAX_DOC_OFFSETS, that can be stored in a record.  In the case of
     overflow (more than the maximum number of doc offsets need to 
     be stored), an overflow record is created at the end of the file 
     and the offset to that record is stored in the Postings record.

     The format of the Postings record is as follows:

     SIZE OF FIELD            FIELD DESCRIPTION

     long                     Number of offsets stored in the record.
     long                     Offset to overflow record.
     long                     Doc offset which is the Key's offset 
                              within Docs file (Note that there may
                              be up to MAX_DOCS_OFFSET doc offsets
                              stored in each record).

     The size of each Postings record is locked-in by the padding of
     zero's, e.g., if MAX_DOCS_OFFSETS = n and the number of doc offsets
     actually stored in the Postings record is x, there will be (n - x)
     fields of size long padded with zeros.

     Each Postings record is created by the function create_posting.
     A Postings record is updated by the function update_postings_file. 

(4)  '*fp_btree' is a pointer to the B-Tree.  The B-Tree is stored in 
     pages of size PAGE_SIZE bytes.  The first page of the B-Tree is the
     root and it is stored as page number ROOT.  A page may be either a
     leaf page or a non-leaf page.  
 
     The format of a non-leaf page is as follows:

     The page header:

       FIELD SIZE            FIELD DESCRIPTION

       char                  Symbol defined as 'N' for "non"-leaf.
       long                  Page number within B-Tree.
       int                   Number of bytes contained within the page.
       int                   Number of keys contained within the page.
       long                  Page number of rightmost child.
   
     Each key:

       FIELD SIZE            FIELD DESCRIPTION 

       long                  Page number of child page containing keys
                               that are lexicographically less than the
                               stored key.
       int                   The length (in bytes) of the stored key.
       <key_len> bytes       The key which may consist of up to
                               MAX_WORD_SIZE bytes.
       ptr (8 bytes)         Pointer to the next key record in the 
                               linked list (or NULL if tail).

     The format of a Leaf page is as follows:

     The page header:

       FIELD SIZE            FIELD DESCRIPTION

       char                  Symbol defined as 'L' for "leaf".
       long                  Page number within B-Tree.
       long                  Page number of next logical leaf page.
       int                   Number of bytes within page.
       int                   Number of keys within page.

     Each key:
 
       FIELD SIZE            FIELD DESCRIPTION

       int                   The length (in bytes) of the stored key.
       <key_len> bytes       The key which may consist of up to
                               MAX_WORD_SIZE bytes.
       long                  The Postings offset which is the offset 
                               of the Postings record.    
       ptr (8 bytes)         Pointer to the next key record in the 
                               linked list (or NULL if tail).

     The size of each B-Tree page is locked-in by the padding of
     character zero's, e.g., if the number of bytes stored in a 
     page is num_bytes = n and the PAGE_SIZE = x, there will be
     (x - n) character bytes padded with '0'.  

(5)  'Makefile' contains a list of all functions referenced by the
     program.  After any program editing, type 'make' at the system
     command line to re-compile the program.  This file currently
     instructs the system to store the compiled program in a file
     called 'main' in the 'bin' folder.  At execution time, type 
     'bin/main' at the system command line to run the program.  

(6)  'common.h' is the definition file for the B-Tree structures. 
     It includes declarations for all program constants, globals,
     typedef declarations, structure definitions, and some helper
     functions.

(7)  'main.c' is the main driver file for the program.  This file
     opens input and output files, and drives the program.


INTERNAL DATA STRUCTURES:

The function fetch_page reads a page from disk using the formula
page_num * PAGE_SIZE - PAGE_SIZE, e.g., if the page_num = 3 and
PAGE_SIZE = 1024, the B-Tree file will be accessed starting at
offset 2048, wherein the first page would be stored at offset 0,
and the second page at offset 1024. 

Function fetch_page installs the page in the following structures:

The dynamically allocated page header structure:

    struct PageHdr {
       char          page_type;             /* To differentiate between  
                                                 Leaf or NonLeaf pages       */
       long          page_num;              /* Page number within the
                                                 B-Tree                      */
       long          next_leaf_page_num;    /* Page number of next logical 
                                                 leaf page (LEAF PAGES ONLY) */
       int           num_bytes;             /* Number of bytes stored
                                                 within page                 */ 
       int           num_keys;              /* Number of keys stored
                                                 within page                 */  
       KeyRecord*    key_list_ptr;          /* Pointer to the list of keys
                                                 and their relative data     */
       long          final_right_page_num;  /* Page number of righmost child   
                                                 (NONLEAF PAGES ONLY         */
    };

The dynamically allocated key structure:

    struct KeyRecord {
       long          page_num;         /* Page number of child page con- 
                                            taining keys lexicograph-
                                            ically less than stored
                                            key (NONLEAF PAGES ONLY)  */
       int           key_len;          /* The length (in bytes) of
                                            the stored key            */ 
       char*         stored_key;       /* A pointer to the dynamically
                                            allocated storage for
                                            the key containing up to
                                            MAX_WORD_SIZE characters    */ 
       long          posting;          /* Offset of Postings record
                                            (LEAF PAGES ONLY)         */
       KeyRecord*    next;             /* Pointer to the next logical
                                            KeyRecord structure       */  
    };

The function flush_page writes a page to disk (again, using the 
formula page_num * PAGE_SIZE - PAGE_SIZE) and calls the function
free_node to deallocate storage space previously allocated
for each key, each KeyRecord structure, and the PageHdr structure.


INPUT:

The program reads a document (as a text file containing a sequence of
words) which is concatenated to the main Docs file.


OUTPUT:

The program can print the contents of the B-tree in a neat in-order format.
It can also print the contents of the B-Tree, page by page, beginning with
the Root.  The pages have been stored and are printed in an ordered manner,
i.e., 1 < 2 < ... < n. 


FUNCTIONALITY:

The source code comprises the following components:
 - common:    structure definitions and global constants and variables
 - btree:     internals related to B-tree manipulation
 - postings:  internals related to postings lists manipulation
 - docs:      internals related to docs file manipulation
 - api:       external interface for creating and using B-tree

Each folder contains a header file of the same name which documents
all the functions implemented by the .c files in that folder.

