/* modified by Frank Andrasco to initialize the first record of postings file */

#include <assert.h>
#include <math.h>
#include "api.h"

/* Global definitions */

/* Database file pointers.  Initialized in db_open() */
FILE* fp_btree;     /* B-tree file pointer */
FILE* fp_postings;  /* Postings file pointer */
FILE* fp_docs;      /* Documents file pointer */

/* Database parameters.  Initialized in set_params() */
int    PAGE_SIZE;       /* Size of page (in bytes) to be stored on disk */
int    MAX_DOCS_PTRS;   /* Postings file stores this many ptrs before overflowing */
double GROWTH_FACTOR;   /* Postings lists growth factor */
long   LIST_SIZE[100];  /* Postings lists sizes */

/*
 * Initialize database parameters from PARAMS_FILE.
 * Reads and sets the PAGE_SIZE and the number of ptrs/postings_record.
 */
void set_params() {
    FILE* fp = fopen(PARAMS_FILE, "r");
    if (fp == NULL) {
        printf("file %s not found\n", PARAMS_FILE);
        exit(-1);
    }

    double gf;
    fscanf(fp, "%d %lf", &PAGE_SIZE, &gf);
    fclose(fp);

    MAX_DOCS_PTRS = 1;  /* to be deleted? */
    GROWTH_FACTOR = gf;
    assert(gf >= 1.5);
    for (int i = 0; i < 100; i++) {
        LIST_SIZE[i] = (long) ceil(pow(gf, (double) i));
#ifdef DEBUG

        printf("ls[%d]=%ld\n", i, LIST_SIZE[i]);
#endif
    }
}

/*
 * Open/create the three store files (BTREE_FILE, POSTINGS_FILE, DOCS_FILE).
 */
void db_open() {

    set_params();

    if ((fp_postings = fopen(POSTINGS_FILE, "r+")) == NULL) {
        printf("Creating postings file (%s) ... \n", POSTINGS_FILE);
        fp_postings = fopen(POSTINGS_FILE, "w+");
        initialize_postings_file();
        printf("Done \n");
    }

    if ((fp_btree = fopen(BTREE_FILE, "r+")) == NULL) {
        printf("Creating B-tree file (%s) ... \n", BTREE_FILE);
        fp_btree = fopen(BTREE_FILE, "w+");
        create_root();  /* an empty root page */
        printf("Done \n");
    }

    if ((fp_docs = fopen(DOCS_FILE, "r+")) == NULL) {
        printf("Creating docs file (%s) ...\n", DOCS_FILE);
        fp_docs = fopen(DOCS_FILE, "w+");
        if (fp_docs == NULL) {
            printf("No write access on docs file - opening for read only\n");
            fp_docs = fopen(DOCS_FILE, "r");
        }
        printf("Done \n");
    }

    printf("Database opened \n");
}

/*
 * Close the three store files (BTREE_FILE, POSTINGS_FILE, DOCS_FILE).
 */
void db_close() {
    fclose(fp_postings);
    fclose(fp_btree);
    fclose(fp_docs);
    printf("Database closed \n");
}
