#include "api.h"

/*
 * This function prints all the pages in the B-tree, page by page, starting with 
 * the Root. The pages have been stored and are printed in an ordered manner, 
 * i.e., 1 < 2 < ... < n.
 */
void print_tree_page_order() {
    printf("Here is the B-Tree By Page (starting with the root):\n\n");
    for (int i = 1; !feof(fp_btree); i++) 
        print_page(i);
}

/*
 * Print the given page (leaf or non-leaf).
 */
void print_page(long page_num) {
    if (page_num < 1) {
        printf("page numbers start from 1 and on\n");
        return;
    }

    if (ffsize(fp_btree) <= (page_num - 1) * PAGE_SIZE) { /* illegal page number */
        printf("page numbers start from 1 and not exceed the last one \n");
        return;
    }

    fseek(fp_btree, (page_num - 1) * PAGE_SIZE, 0);
    
    char c;
    fread(&c, sizeof(c), 1, fp_btree);
    if (feof(fp_btree) != 0)
        exit(0);
    printf("Leafstatus:%c\n", c);
    
    long page_num_;
    fread(&page_num_, sizeof(page_num_), 1, fp_btree);
    printf("Page#:%d\n", (int) page_num_);
    
    if (c == LEAF_SYMBOL) {
        long next_leaf_page_ptr;
        fread(&next_leaf_page_ptr, sizeof(next_leaf_page_ptr), 1, fp_btree);
        printf("PtrtoNextLeafPage:%d\n", (int) next_leaf_page_ptr);
    }
    
    int num_bytes, num_keys;
    fread(&num_bytes, sizeof(num_bytes), 1, fp_btree);
    printf("Number of bytes occcupied:%d\n", num_bytes);
    fread(&num_keys, sizeof(num_keys), 1, fp_btree);
    printf("Number of keys:%d\n", num_keys);
    
    if (c == NON_LEAF_SYMBOL) {
        long final_right_page_num;
        fread(&final_right_page_num, sizeof(final_right_page_num), 1, fp_btree);
        printf("final_right_page_num%d\n", (int) final_right_page_num);
    }
    
    char key[MAX_WORD_SIZE];
    for (int j = 0; j < num_keys; j++) {
        if (c == NON_LEAF_SYMBOL) {
            fread(&page_num_, sizeof(page_num_), 1, fp_btree);
            printf("NextPage:%d, ", (int) page_num_);
            int key_len;
            fread(&key_len, sizeof(key_len), 1, fp_btree);
            fread(key, sizeof(char), key_len, fp_btree);
            (*(key + key_len)) = '\0';
            printf("key:%s, ", key);
            printf("length:%d\n", key_len);
        }
        if (c == LEAF_SYMBOL) {
            int key_len;
            fread(&key_len, sizeof(key_len), 1, fp_btree);
            printf("Keylength:%d, ", key_len);
            fread(key, sizeof(char), key_len, fp_btree);
            (*(key + key_len)) = '\0';
            printf("key:%s, ", key);
            long postings_offset;
            fread(&postings_offset, sizeof(postings_offset), 1, fp_btree);
            printf("Postings offset:%d\n", (int) postings_offset);
        }
    }
    printf("\n");
}

/* 
 * Recursively print the pages in the B-tree via an in-order traversal,
 * from the given page_num onwards. Call it on the root page with depth 0.
 */
void print_tree_inorder(long page_num, int depth) {
    PageHdr* p = fetch_page(page_num);
    
    if (!is_leaf(p)) {
        for (KeyRecord* k = p->key_list_ptr; k != NULL; k = k->next) {
            print_tree_inorder(k->page_num, depth + 1);
            for (int i = 0; i < depth * 15; i++) {
                printf(" ");
            }
            printf("%s \n", k->stored_key);
        }
        print_tree_inorder(p->final_right_page_num, depth + 1);
        
    } else {
        int count = 0;
        for (KeyRecord* k = p->key_list_ptr; k != NULL; k = k->next) {
            count++;
            if (k == p->key_list_ptr) {
                for (int i = 0; i < depth * 15; i++) {
                    printf(" ");
                }
                printf("%s ", k->stored_key);
            } else if (k->next == NULL) {
                printf("... %s ", k->stored_key);
            }
        }
        printf("(%d) \n", count);
    }
}

