#include "btree.h"

/*
 * This recursive function searches a list of keys to find the
 * position where the new key should be inserted. The new key is
 * compared with the keys stored in the B-Tree until (1) a match is
 * found, (2) the new key is lexicographically less than the stored
 * key, (3) the last stored key is encountered and conditions (1)
 * or (2) have not been met. The function returns, for the above
 * cases: Case (1) -- the position of the matched key, Case (2) --
 * the position immediately preceding the current key being
 * compared, or Case (3) -- the position of the last key.
 *
 * Note that the variable count starts out with value zero and
 * increments with each recursive call.
 *
 * The variable key_list_iterator is a pointer that traverses
 * the list of keys.
 */
int find_insertion_pos(KeyRecord* key_list_iterator, char* key,
                       int* found, int num_keys, int count) {
    /* -christos- the next block probably provides for insertion in empty list
     * (useful for insertion in root for the first time! */

    if (num_keys == 0) {
        *found = FALSE;
        return count;
    }

    /* Compare the the possible new key with the key stored in B-Tree */
    int result = compare_keys(key, key_list_iterator->stored_key);

    num_keys = num_keys - 1;
    count    = count + 1;

    if (result == 0) {  /* Match found */
        *found = TRUE;
        return (count);  /* Location of matched key */
    } else  /* No match yet */
        *found = FALSE;

    if (num_keys > 0) {
        if (result == 1)  /* New key < stored key */
            return (count - 1);  /* Location before stored key */
        else {  /* New key > stored key: keep searching */
            key_list_iterator = key_list_iterator->next;
            return (find_insertion_pos(key_list_iterator, key, found, num_keys, count));
        }

    } else {  /* this is the last key in the list -- search must terminate */
        if (result == 1)  /* New key < stored key */
            return (count - 1);  /* Location before stored key */
        else
            return (count);  /* New key will be the last key */
    }
}

