#include "btree.h"
#include "../postings/postings.h"

/*
 * This function inserts a key into a leaf page, after a posting has
 * been created. In the case where a key is already stored in the
 * B-Tree, the posting record is updated.
 *
 * If the page overflows, it splits it, flushes the two new pages and
 * return the value of the middle key in a structure "UpKey"
 * Otherwise, it just flushes back the old page and returns NULL
 */
UpKey* insert_in_leaf(PageHdr* page_ptr, char* key, long doc_offset) {
    /* Find insertion position */
    KeyRecord* key_list_iterator = page_ptr->key_list_ptr;
    int count = 0, found;
    int insert_pos = find_insertion_pos(key_list_iterator, key, 
                                        &found, page_ptr->num_keys, count);

    /* key is already in the B-Tree */
    if (found == TRUE) {

        long oldpointer;
        /* printf ("key = %s\n", key);
        printf (" Text Offsets are:\n"); */
        for (int i = 0; i < insert_pos - 1; i++)
            key_list_iterator = key_list_iterator->next;
        /* change made here by Frank Andrasco
        since postings pointer may change, pointer must be passed by reference
        instead of by value */

        oldpointer = key_list_iterator->posting;
        update_postings_file(&key_list_iterator->posting, doc_offset);
        /* check to see if posting value changed, if so rewrite to disk */
        if (key_list_iterator->posting == oldpointer)
            free_page(page_ptr);
        else
            flush_page(page_ptr);
        return (NULL);
    }

    /* key must be inserted in B-Tree */
    long post_offset;
    create_posting(doc_offset, &post_offset);
    /* Traverse the list of keys to insertion position */
    key_list_iterator = page_ptr->key_list_ptr;
    KeyRecord* key_list_trailer;
    for (int i = 0; i < insert_pos; i++) {
        key_list_trailer  = key_list_iterator;
        key_list_iterator = key_list_iterator->next;
    }

    /* Create a new node for the new key and insert key informtion */
    KeyRecord* new_key_record = (KeyRecord*) malloc(sizeof(KeyRecord));
    check_malloc(new_key_record, "new_key_record");
    new_key_record->key_len    = strlen(key);
    new_key_record->stored_key = strdup(key);
    new_key_record->posting    = post_offset;

    /* Insert new key in list */
    if (insert_pos == 0) {
        page_ptr->key_list_ptr = new_key_record;
    } else
        key_list_trailer->next = new_key_record;
    /* Link new key to keys that lexicographically follow it */
    new_key_record->next = key_list_iterator;

    /* Update page header information */
    update_page_header(page_ptr);

    /* split page, if necessary, and flush */
    if (page_ptr->num_bytes <= PAGE_SIZE) {
        flush_page(page_ptr);
        return NULL;
    } else {
        UpKey* middle_key = split_page(page_ptr);
        return middle_key;
    }
}
