#include "btree.h"

/*
 * This function inserts a key into a nonleaf page. In the case
 * where the nonleaf page contains zero keys (the root), the first
 * key is inserted into the root and the only leaf page becomes
 * the rightmost child.
 * 
 * It returns NULL, or a pointer to the key ("UpKey") to be propagated up.
 */
UpKey* insert_in_non_leaf(PageHdr* page_ptr, UpKey* middle_key) {
    if (middle_key == NULL) { /* just free the page-image */
        free_page(page_ptr);
        return (NULL);
    }

    /* build an appropriate KeyRecord node for the inserted key */
    KeyRecord* middle_key_copy  = (KeyRecord*) malloc(sizeof(KeyRecord));
    middle_key_copy->page_num   = middle_key->left;
    middle_key_copy->key_len    = strlen(middle_key->key);
    middle_key_copy->stored_key = strdup(middle_key->key);

    /* Root page currently has zero keys: this key will be first insertion */
    /* NOTE: maybe this "if" block is redundant, if "find_insertion_pos" */
    /* works with a empty list!: -christos- */
    if (page_ptr->num_keys == 0) {
        page_ptr->final_right_page_num = middle_key->right;
        page_ptr->key_list_ptr         = middle_key_copy;
        middle_key_copy->next = NULL;
        flush_page(page_ptr);  /* it fills in the # of bytes & # of keys */
        return (NULL);
    }

    /* Find insertion position for key */
    /* -christos- : make it a procedure, say, inList() */
    char* word = middle_key_copy->stored_key;
    int count = 0, found;
    KeyRecord* key_list_iterator = page_ptr->key_list_ptr;
    int insert_pos = find_insertion_pos(key_list_iterator, word,
                                        &found, page_ptr->num_keys, count);

    /* Traverse list of keys to insertion postion */
    key_list_iterator = page_ptr->key_list_ptr;
    KeyRecord* key_list_trailer;  /* Pointers to the list of keys */
    for (int i = 0; i < insert_pos; i++) {
        key_list_trailer  = key_list_iterator;
        key_list_iterator = key_list_iterator->next;
    }

    /* Insert the key */
    if (insert_pos == 0) /* This will be first key in list */
        page_ptr->key_list_ptr = middle_key_copy;
    else
        key_list_trailer->next = middle_key_copy;

    /* Link key to keys that lexicographically follow it */
    middle_key_copy->next = key_list_iterator;

    /* Update the page header and, possibly, the pagepointer of the next key */
    if (middle_key_copy->next == NULL) { /* ie., it is last */
        page_ptr->final_right_page_num = middle_key->right;
    } else {
        middle_key_copy->next->page_num = middle_key->right;
    }
    update_page_header(page_ptr);  /* # bytes & # keys */

    free(middle_key->key);
    free(middle_key);

    /* split, if it overflows, and flush */
    if (page_ptr->num_bytes <= PAGE_SIZE) {
        flush_page(page_ptr);
        return NULL;
    } else {
        UpKey* new_up_key = split_page(page_ptr);
        return new_up_key;
    }
}
