#include "btree.h"

/*
 * This function inserts a (key, offset) entry in the B-tree, even if it is empty initially. 
 * If the B-Tree does exist, the key is sent to propagated_insert for insertion and any
 * possible page splits are handled there by recursively propagating down and then up the tree. 
 * If propagated_insert returns the value NULL, this function terminates, otherwise
 * the root must be split (split_root()).
 */
void insert_in_tree(char* key, long doc_offset) {
    /* Print an error message if the key is too long */
    int key_len = strlen(key);
    if ((key_len > MAX_WORD_SIZE)
        || (key_len > (PAGE_SIZE - sizeof(PageHdr) - sizeof(long) - sizeof(int)))) {
        printf("ERROR: key is too long-operation aborted\n");
        printf("offending key:\"%s\"\n", key);
        return;
    }

    /* insert key */
    UpKey* middle_key = propagated_insert(ROOT, key, doc_offset);

    /* The Root Must be Split */
    if (middle_key != NULL)
        split_root(ROOT, middle_key);
}
