#include "btree.h"

/*
 * Prior to execution of this function, the Root has already been
 * split by the function split_page(). The current Root contains
 * the left half of the Root that was split. This function creates
 * a new root (still located as the first page on disk) and installs
 * the key (up_key_ptr) that has propagated up to the Root. 
 * The current root is installed as the left child of the new Root.
 */
void split_root(long root_page_num, UpKey* up_key_ptr) {
    PageHdr* root_page_ptr;  /* Pointer to page (Root) being split */
    PageHdr* new_page_ptr;   /* Pointer to newly created page */

    KeyRecord* middleKey;  /* holds the key that will be the only key of the new root */

    /* allocate space for the *middlekey node and fill it in */
    middleKey = (KeyRecord*) malloc(sizeof(KeyRecord));
    check_malloc(middleKey, "middleKey");
    middleKey->stored_key = strdup(up_key_ptr->key);
    middleKey->key_len    = strlen(middleKey->stored_key);
    middleKey->next       = NULL;

    /* read in root */
    root_page_ptr = fetch_page(ROOT);

    /* Create the core-image of the new root page */
    new_page_ptr = (PageHdr*) malloc(sizeof(*new_page_ptr));
    check_malloc(new_page_ptr, "new_page_ptr");
    new_page_ptr->page_type            = NON_LEAF_SYMBOL;
    new_page_ptr->page_num             = ROOT;
    new_page_ptr->final_right_page_num = up_key_ptr->right;
    /* points to the second half of old root */

    /* Record the page pointers */
    root_page_ptr->page_num = get_new_page_num();
    /* the left part of the old root will be flushed */
    /* at a new position */

    middleKey->page_num = root_page_ptr->page_num;

    /* Install the middle key into the new root */
    new_page_ptr->key_list_ptr = middleKey;
    /* - christos -: free the space of the UpKey - DONE! */
    free(up_key_ptr->key);
    free(up_key_ptr);

    flush_page(new_page_ptr);
    flush_page(root_page_ptr);
}

