/*
 * This is the definition file for the B-Tree program. It contains
 * declarations for all functions utilized; program macros and
 * constants; a few necessary globals; typedef declarations; and
 * structure definitions.
 *
 * Consult the Programmer's Manual ("Pgmrs.Man") for the program
 * description.
 */

#ifndef COMMON_H
#define COMMON_H

#include "constants.h"

/* Forward declarations and typedefs of structs */
typedef struct PageHdr   PageHdr;
typedef struct KeyRecord KeyRecord;
typedef struct UpKey     UpKey;

/*
 * The following structure is utilized for holding a page of the B-Tree.
 * It is used for both Leaf and NonLeaf pages. The pages are differentiated
 * by the first byte which contains 'L' if it is a Leaf page or 'N' if it
 * is a NonLeaf page. All Leaf pages contain the field 'next_leaf_page_num'
 * which is a pointer (page number) to the next logical leaf page. All
 * NonLeaf pages contain the field 'final_right_page_num' which is a pointer
 * (page number) to the rightmost child. 
 */
struct PageHdr {
    char page_type;  /* 'N' for NonLeaf, 'L' for Leaf */
    long page_num;
    long next_leaf_page_num;    /* FOR LEAF PAGES ONLY */
    int  num_bytes;
    int  num_keys;
    KeyRecord* key_list_ptr;
    long final_right_page_num;  /* FOR NONLEAF PAGES ONLY */
};

static inline int is_leaf(PageHdr* x) { return x->page_type == LEAF_SYMBOL; }

/*
 * The following structure is used to hold the keys which are
 * stored in the B-Tree page. It is used for both Leaf and NonLeaf keys.
 * If the page is a NonLeaf page, the key will be accompanied by the
 * field 'page_num' which is a pointer (page number) to a left child page
 * that contains keys which are lexicographically less than the the key
 * in this page. If the page is a Leaf page, the key will be accompanied
 * by the field 'posting' which is a pointer (offset) into the
 * POSTINGS_FILE (which contains the offsets into the DOCS_FILE).
 */
struct KeyRecord {
    long  page_num;  /* FOR NONLEAF PAGES ONLY */
    int   key_len;
    char* stored_key;
    long  posting;   /* FOR LEAF PAGES ONLY */
    KeyRecord* next;
};

/*
 * 'UpKey' should be used to hold the key to be moved upwards upon splitting.
 */
struct UpKey {
    long  left;   /* left page, with keys <= */
    long  right;  /* right page, with keys > */
    char* key;
};

#include "helpers.h"
#include "globals.h"

#endif