/*
 * Main driver for btree-based text retreival system.
 */

/*
 * Modified by Yousuf Ahmad, March 2018.
 * - major refactoring
 */

/*
 * modified by christos, Sept. 2009,
 * - to check for input buffer overflows
 * - to check for page-numbers out of range in print_page()
 */

/*
 * modified by Sanjay, Oct 2000,
 * - Commented out options "b","d"; added new option T for inorder traversal
 */

/*
 * modified by christos, Nov. 23 '91
 */

/*
 * modified by Frank Andrasco 1990
 */

#include <assert.h>
#include <string.h>
#include "api/api.h"

int main() {

    db_open();     /* opens or creates the three files (btree, postings, text) */

    int loop = TRUE;
    while (loop) {

        printf("\n\t* These are your commands .........\n");
        printf("\t\"C\" to scan the B-tree\n");
        printf("\t\"i\" to insert a document into the B-tree\n");
        printf("\t\"f\" to print, in alphabetical order, all the distinct keys "
               "between (and including) key1 and key2\n");
        printf("\t\"b\" to print, in reverse alphabetical order, all the "
               "distinct keys between (and including) key1 and key2\n");
        printf("\t\"p\" to print a B-tree page\n");
        printf("\t\"s\" to search, and print the key\n");
        printf("\t\"S\" to search, and print the <key, postings_list> pairs\n");
        printf("\t\"T\" to print the B-tree in in-order format\n");
        printf("\t\"q\" to exit\n");

        char word[MAX_WORD_SIZE];
        char word2[MAX_WORD_SIZE];

        char cmd[MAX_WORD_SIZE];    /* string to hold a command */
        scanf("%s", cmd);
        assert(strlen(cmd) < MAX_WORD_SIZE);

        switch (cmd[0]) {
            case 'C':printf("\n* Scanning... \n");
                scan_tree_leafs(&print_leaf_key_record);
                break;
            case 'i':printf("\tgive input file name: ");
                char fname[MAX_WORD_SIZE];  /* name of input file */
                scanf("%s", fname);
                assert(strlen(fname) < MAX_WORD_SIZE);
                printf("\n* Inserting %s\n", fname);
                insert(fname);
                break;
            case 'f':printf("enter first keyword: ");
                scanf("%s", word);
                assert(strlen(word) < MAX_WORD_SIZE);
                printf("\nenter second keyword: ");
                scanf("%s", word2);
                assert(strlen(word2) < MAX_WORD_SIZE);
                printf("\n* Finding the keywords between %s and %s (including bounds).\n",
                       word, word2);
                range_search(word, word2);
                break;
            case 'b':printf("enter first keyword: ");
                scanf("%s", word);
                assert(strlen(word) < MAX_WORD_SIZE);
                printf("\nenter second keyword: ");
                scanf("%s", word2);
                assert(strlen(word2) < MAX_WORD_SIZE);
                printf("\n* Finding the keywords between %s and %s (including bounds)"
                       "in reverse alphabetical order.\n", word, word2);
                reverse_range_search(word, word2);
                break;
            case 's':printf("enter search-word: ");
                scanf("%s", word);
                assert(strlen(word) < MAX_WORD_SIZE);
                printf("\n* Searching for word %s \n", word);
                search(word, FALSE);
                break;
            case 'S':printf("enter search-word: ");
                scanf("%s", word);
                assert(strlen(word) < MAX_WORD_SIZE);
                printf("\n* Searching for word %s \n", word);
                search(word, TRUE);
                break;
            case 'p':printf("pagenumber=?\n");
                scanf("%s", cmd);
                assert(strlen(cmd) < MAX_WORD_SIZE);
                print_page(atol(cmd));
                break;
            case 'T':printf("\n* Printing tree in order .........\n");
                print_tree_inorder(ROOT, 0);
                break;
            case 'q':printf("\n* Exiting .........\n");
                loop = FALSE;
                break;
            default:
                printf("\n* Illegal command \"%s\"\n", cmd);
                break;
        }
    }

    db_close();

    /* print_tree_page_order(); */
    return 0;
}
